@@ -46,27 +46,7 @@ namespace ranges
 {
     inline namespace v3
     {
-        inline namespace function_objects {}
-
-        namespace aux
-        {
-            inline namespace function_objects {}
-        }
-
-        namespace view
-        {
-            inline namespace function_objects {}
-        }
-
-        namespace action
-        {
-            inline namespace function_objects {}
-        }
-
-        namespace detail
-        {
-            inline namespace function_objects {}
-        }
+        inline namespace CPOs {}
 
         /// \cond
         namespace _end_
@@ -329,9 +309,6 @@ namespace ranges
         #endif
 
             // Work around libc++'s buggy std::is_function
-        #if !defined(_LIBCPP_VERSION) || _LIBCPP_VERSION >= 3800
-            using std::is_function;
-        #else
             // Function types here:
             template<typename T>
             char (&is_function_impl_(priority_tag<0>))[1];
@@ -353,7 +330,6 @@ namespace ranges
             struct is_function
               : meta::bool_<sizeof(detail::is_function_impl_<T>(priority_tag<3>{})) == 1>
             {};
-        #endif
 
             template<typename T>
             struct remove_rvalue_reference
@@ -421,14 +397,8 @@ namespace ranges
         template<typename Cur>
         struct basic_mixin;
 
-        /// \cond
-        namespace _basic_iterator_
-        {
-            template<typename Cur>
-            struct basic_iterator;
-        }
-        using _basic_iterator_::basic_iterator;
-        /// \endcond
+        template<typename Cur>
+        struct basic_iterator;
 
         template<cardinality>
         struct basic_view : view_base
@@ -442,14 +412,8 @@ namespace ranges
                  cardinality C = range_cardinality<BaseRng>::value>
         struct view_adaptor;
 
-        /// \cond
-        namespace _common_iterator_
-        {
-            template<typename I, typename S>
-            struct common_iterator;
-        }
-        using _common_iterator_::common_iterator;
-        /// \endcond
+        template<typename I, typename S>
+        struct common_iterator;
 
         template<typename I, typename S>
         using common_iterator_t =
@@ -536,12 +500,6 @@ namespace ranges
 
         struct default_sentinel { };
 
-        namespace _counted_iterator_
-        {
-            template<typename I, typename = void>
-            struct counted_iterator;
-        }
-
         template<typename I>
         struct move_iterator;
 
